----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 10/26/2022 03:16:19 AM
-- Design Name: 
-- Module Name: quickstart - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
USE IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

ENTITY quickstart IS
    PORT (
        clk_100mhz : IN STD_LOGIC; --clock
        sw : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
        btnc : IN STD_LOGIC;
        btnu : IN STD_LOGIC;
        btnl : IN STD_LOGIC;
        btnr : IN STD_LOGIC;
        btnd : IN STD_LOGIC;
        led : OUT STD_LOGIC_VECTOR(15 DOWNTO 0); -- out leds
        RGB1_Red : OUT STD_LOGIC;
        RGB1_Green : OUT STD_LOGIC;
        RGB1_Blue : OUT STD_LOGIC;
        RGB2_Red : OUT STD_LOGIC;
        RGB2_Green : OUT STD_LOGIC;
        RGB2_Blue : OUT STD_LOGIC
    );

END quickstart;

ARCHITECTURE Behavioral OF quickstart IS
    CONSTANT MAX_COUNT : INTEGER := 25000000; --turn on/off LED every 250 ms...so flash at 2 Hz
    SIGNAL counter : unsigned(31 DOWNTO 0); -- for flashing the LEDs
    SIGNAL dim_counter : unsigned(15 DOWNTO 0); -- for adjusting LED brightness
    SIGNAL rgb_enable : STD_LOGIC; --used for enabling LED
    SIGNAL dim_on_off : STD_LOGIC; -- used for PWM-dimming the RGB LEDs

BEGIN

    --do the combinatorial stuff first:
    led <= sw; --blanket assignment of all 16 switch values to all 16 LEDs
    dim_on_off <= '1' WHEN dim_counter < unsigned(sw) ELSE
        '0'; --the higher the switches, the longer this will be on for a cycle.
    -- WARNING : it is BRIGHT !

    -- make the RGB LEDs be based on buttons (for color) and switches (for brightness)
    RGB1_Blue <= dim_on_off AND (rgb_enable OR btnl);
    RGB1_Green <= dim_on_off AND (rgb_enable OR btnc);
    RGB1_Red <= dim_on_off AND (rgb_enable OR btnr);
    RGB2_Blue <= dim_on_off AND (rgb_enable OR btnu);
    RGB2_Green <= dim_on_off AND (rgb_enable OR btnc);
    RGB2_Red <= dim_on_off AND (rgb_enable OR btnd);

    PROCESS (clk_100mhz)
    BEGIN
        IF rising_edge(clk_100mhz) THEN
            IF (to_integer(counter) = MAX_COUNT) THEN
                counter <= (OTHERS => '0');
                rgb_enable <= NOT rgb_enable;
            ELSE
                counter <= counter + 1;
            END IF;
            dim_counter <= dim_counter + 1;
        END IF;
    END PROCESS;

END Behavioral;